@@ -54,10 +54,10 @@
  *	software in accordance with the terms specified in this
  *	license.
  *
- * RCS: @(#) $Id: tkMacOSXWindowEvent.c,v 1.3.2.19 2007/05/09 12:57:47 das Exp $
+ * RCS: @(#) $Id: tkMacOSXWindowEvent.c,v 1.3.2.26 2007/11/09 06:26:56 das Exp $
  */
 
-#include "tkMacOSXInt.h"
+#include "tkMacOSXPrivate.h"
 #include "tkMacOSXWm.h"
 #include "tkMacOSXEvent.h"
 #include "tkMacOSXDebug.h"
@@ -68,24 +68,15 @@
 #endif
 */
 
-/*
- * Declarations of global variables defined in this file.
- */
-
-static int tkMacOSXAppInFront = true;	/* Boolean variable for determining if
-					 * we are the frontmost app. Only set
-					 * in TkMacOSXProcessApplicationEvent
-					 */
-static RgnHandle gDamageRgn = NULL;
-
 /*
  * Declaration of functions used only in this file
  */
 
-static int GenerateUpdateEvent( Window window);
-static int GenerateUpdates( RgnHandle updateRgn, TkWindow *winPtr);
-static int GenerateActivateEvents( Window window, int activeFlag);
-static void ClearPort(CGrafPtr port);
+static int GenerateUpdateEvent(Window window);
+static int GenerateUpdates(HIMutableShapeRef updateRgn, CGRect *updateBounds,
+	TkWindow *winPtr);
+static int GenerateActivateEvents(Window window, int activeFlag);
+static void ClearPort(CGrafPtr port, HIShapeRef updateRgn);
 
 
 /*
@@ -100,7 +91,7 @@ static void ClearPort(CGrafPtr port);
  *	0.
  *
  * Side effects:
- *	Hide or reveal floating windows, and set tkMacOSXAppInFront.
+ *	Hide or reveal floating windows.
  *
  *----------------------------------------------------------------------
  */
@@ -122,12 +113,10 @@ TkMacOSXProcessApplicationEvent(
 
     switch (eventPtr->eKind) {
 	case kEventAppActivated:
-	    tkMacOSXAppInFront = true;
 	    ShowFloatingWindows();
 	    break;
 	case kEventAppDeactivated:
 	    TkSuspendClipboard();
-	    tkMacOSXAppInFront = false;
 	    HideFloatingWindows();
 	    break;
 	case kEventAppQuit:
@@ -243,7 +232,7 @@ TkMacOSXProcessWindowEvent(
 	case kEventWindowActivated:
 	case kEventWindowDeactivated:
 	case kEventWindowUpdate:
-	case kEventWindowExpanded:
+	case kEventWindowExpanding:
 	case kEventWindowBoundsChanged:
 	case kEventWindowDragStarted:
 	case kEventWindowDragCompleted:
@@ -286,10 +275,20 @@ TkMacOSXProcessWindowEvent(
 		statusPtr->stopProcessing = 1;
 	    }
 	    break;
-	case kEventWindowExpanded:
+	case kEventWindowExpanding:
 	    if (winPtr) {
-		TkpWmSetState(winPtr, TkMacOSXIsWindowZoomed(winPtr) ?
-			ZoomState : NormalState);
+		winPtr->wmInfoPtr->hints.initial_state =
+			TkMacOSXIsWindowZoomed(winPtr) ? ZoomState : 
+			NormalState;
+		Tk_MapWindow((Tk_Window) winPtr);
+		/*
+		 * Need to process all Tk events generated by Tk_MapWindow()
+		 * before returning to ensure all children are mapped, as
+		 * otherwise the Activate event that follows Expanding would
+		 * not be processed by any unmapped children.
+		 */
+		while (Tcl_ServiceEvent(TCL_WINDOW_EVENTS)) {};
+		while (Tcl_DoOneEvent(TCL_IDLE_EVENTS)) {};
 	    }
 	    break;
 	case kEventWindowBoundsChanged:
@@ -315,6 +314,9 @@ TkMacOSXProcessWindowEvent(
 		    height = bounds.bottom - bounds.top;
 		    flags |= TK_SIZE_CHANGED;
 		}
+		TkMacOSXInvalClipRgns((Tk_Window) winPtr);
+		TkMacOSXInvalidateWindow((MacDrawable *) window,
+			TK_PARENT_WINDOW);
 		TkGenWMConfigureEvent((Tk_Window)winPtr, x, y, width,
 			height, flags);
 		if (attr & kWindowBoundsChangeUserResize ||
@@ -333,10 +335,10 @@ TkMacOSXProcessWindowEvent(
 	    }
 	    break;
 	case kEventWindowDragStarted:
-	    TkMacOSXTrackingLoop(1);
 	    if (!(TkMacOSXModifierState() & cmdKey)) { 
 		TkMacOSXBringWindowForward(whichWindow);
 	    }
+	    TkMacOSXTrackingLoop(1);
 	    break;
 	case kEventWindowDragCompleted: {
 	    Rect maxBounds, bounds, strWidths;
@@ -411,7 +413,7 @@ TkMacOSXProcessWindowEvent(
 		CGrafPtr port;
 
 		GetPort(&port);
-		ClearPort(port);
+		ClearPort(port, NULL);
 	    }
 	    break;
     }
@@ -438,11 +440,14 @@ TkMacOSXProcessWindowEvent(
 static int
 GenerateUpdateEvent(Window window)
 {
-    CGrafPtr	destPort;
-    WindowRef	macWindow;
-    TkDisplay * dispPtr;
-    TkWindow  * winPtr;
-    int		result = 0;
+    WindowRef macWindow;
+    TkDisplay *dispPtr;
+    TkWindow  *winPtr;
+    int result = 0;
+    CGRect updateBounds;
+    HIShapeRef rgn;
+    HIMutableShapeRef updateRgn;
+    int dx, dy;
 
     dispPtr = TkGetDisplayList();
     winPtr = (TkWindow *)Tk_IdToWindow(dispPtr->display, window);
@@ -450,31 +455,45 @@ GenerateUpdateEvent(Window window)
     if (winPtr ==NULL ){
 	return result;
     }
-    if (gDamageRgn == NULL) {
-	gDamageRgn = NewRgn();
-    }
-    TkMacOSXCheckTmpRgnEmpty(1);
-    destPort = TkMacOSXGetDrawablePort(window);
-    macWindow = GetWindowFromPort(destPort);
-    GetWindowRegion(macWindow, kWindowUpdateRgn, tkMacOSXtmpRgn1);
-    QDGlobalToLocalRegion(destPort, tkMacOSXtmpRgn1);
-    SectRegionWithPortVisibleRegion(destPort, tkMacOSXtmpRgn1);
+    macWindow = TkMacOSXDrawableWindow(window);
+    TK_IF_MAC_OS_X_API (5, HIWindowCopyShape,
+	ChkErr(HIWindowCopyShape, macWindow, kWindowUpdateRgn,
+		kHICoordSpaceWindow, &rgn);
+	dx = -winPtr->wmInfoPtr->xInParent;
+	dy = -winPtr->wmInfoPtr->yInParent;
+    ) TK_ELSE_MAC_OS_X (5,
+	Rect bounds;
+
+	TkMacOSXCheckTmpQdRgnEmpty();
+	ChkErr(GetWindowRegion, macWindow, kWindowUpdateRgn, tkMacOSXtmpQdRgn);
+	rgn = HIShapeCreateWithQDRgn(tkMacOSXtmpQdRgn);
+	SetEmptyRgn(tkMacOSXtmpQdRgn);
+	ChkErr(GetWindowBounds, macWindow, kWindowContentRgn, &bounds);
+	dx = -bounds.left;
+	dy = -bounds.top;
+    ) TK_ENDIF
+    updateRgn = HIShapeCreateMutableCopy(rgn);
+    CFRelease(rgn);
+    ChkErr(HIShapeOffset, updateRgn, dx, dy);
+    HIShapeGetBounds(updateRgn, &updateBounds);
 #ifdef TK_MAC_DEBUG_CLIP_REGIONS
-    TkMacOSXInitNamedDebugSymbol(HIToolbox, int, QDDebugFlashRegion,
-				CGrafPtr port, RgnHandle region);
-    if (QDDebugFlashRegion) {
-	/* Carbon-internal region flashing SPI (c.f. Technote 2124) */
-	QDDebugFlashRegion(destPort, tkMacOSXtmpRgn1);
-    }
+    TkMacOSXDebugFlashRegion(window, updateRgn);
 #endif /* TK_MAC_DEBUG_CLIP_REGIONS */
     BeginUpdate(macWindow);
     if (winPtr->wmInfoPtr->flags & WM_TRANSPARENT) {
-	ClearPort(destPort);
+	ClearPort(TkMacOSXGetDrawablePort(window), updateRgn);
     }
-    result = GenerateUpdates(tkMacOSXtmpRgn1, winPtr);
+    result = GenerateUpdates(updateRgn, &updateBounds, winPtr);
     EndUpdate(macWindow);
-    SetEmptyRgn(tkMacOSXtmpRgn1);
-    SetEmptyRgn(gDamageRgn);
+    CFRelease(updateRgn);
+    if (result) {
+	/*
+	 * Ensure there are no pending idle-time redraws that could prevent
+	 * the just posted Expose events from generating new redraws.
+	 */
+
+	Tcl_DoOneEvent(TCL_IDLE_EVENTS|TCL_DONT_WAIT);
+    }
     return result;
  }
 
@@ -499,52 +518,52 @@ GenerateUpdateEvent(Window window)
 
 static int
 GenerateUpdates(
-    RgnHandle updateRgn,
+    HIMutableShapeRef updateRgn,
+    CGRect *updateBounds,
     TkWindow *winPtr)
 {
     TkWindow *childPtr;
     XEvent event;
-    Rect bounds, updateBounds, damageBounds;
+    CGRect bounds, damageBounds;
+    HIShapeRef boundsRgn, damageRgn;
 
-    TkMacOSXWinBounds(winPtr, &bounds);
-    GetRegionBounds(updateRgn,&updateBounds);
-
-    if (bounds.top > updateBounds.bottom ||
-	updateBounds.top > bounds.bottom ||
-	bounds.left > updateBounds.right ||
-	updateBounds.left > bounds.right ||
-	!RectInRgn(&bounds, updateRgn)) {
+    TkMacOSXWinCGBounds(winPtr, &bounds);
+    if (!CGRectIntersectsRect(bounds, *updateBounds)) {
 	return 0;
     }
-    if (!RectInRgn(&bounds, updateRgn)) {
-	return 0;
-    }
-
-    event.xany.serial = Tk_Display(winPtr)->request;
-    event.xany.send_event = false;
-    event.xany.window = Tk_WindowId(winPtr);
-    event.xany.display = Tk_Display(winPtr);
-
-    event.type = Expose;
+    TK_IF_MAC_OS_X_API (4, HIShapeIntersectsRect,
+	if (!HIShapeIntersectsRect(updateRgn, &bounds)) {
+	    return 0;
+	}
+    ) TK_ENDIF
 
     /*
      * Compute the bounding box of the area that the damage occured in.
      */
 
-    /*
-     * CopyRgn(TkMacOSXVisableClipRgn(winPtr), rgn);
-     * TODO: this call doesn't work doing resizes!!!
-     */
-    RectRgn(gDamageRgn, &bounds);
-    SectRgn(gDamageRgn, updateRgn, gDamageRgn);
-    OffsetRgn(gDamageRgn, -bounds.left, -bounds.top);
-    GetRegionBounds(gDamageRgn,&damageBounds);
-    event.xexpose.x = damageBounds.left;
-    event.xexpose.y = damageBounds.top;
-    event.xexpose.width = damageBounds.right-damageBounds.left;
-    event.xexpose.height = damageBounds.bottom-damageBounds.top;
-    event.xexpose.count = 0;
+    boundsRgn = HIShapeCreateWithRect(&bounds);
+    damageRgn = HIShapeCreateIntersection(updateRgn, boundsRgn);
+    if (HIShapeIsEmpty(damageRgn)) {
+	CFRelease(damageRgn);
+	CFRelease(boundsRgn);
+	return 0;
+    }
+    HIShapeGetBounds(damageRgn, &damageBounds);
+    ChkErr(TkMacOSHIShapeUnion, boundsRgn, updateRgn, updateRgn);
+    HIShapeGetBounds(updateRgn, updateBounds);
+    CFRelease(damageRgn);
+    CFRelease(boundsRgn);
 
+    event.xany.serial = Tk_Display(winPtr)->request;
+    event.xany.send_event = false;
+    event.xany.window = Tk_WindowId(winPtr);
+    event.xany.display = Tk_Display(winPtr);
+    event.type = Expose;
+    event.xexpose.x = damageBounds.origin.x - bounds.origin.x;
+    event.xexpose.y = damageBounds.origin.y - bounds.origin.y;
+    event.xexpose.width = damageBounds.size.width;
+    event.xexpose.height = damageBounds.size.height;
+    event.xexpose.count = 0;
     Tk_QueueWindowEvent(&event, TCL_QUEUE_TAIL);
 
     /*
@@ -552,12 +571,11 @@ GenerateUpdates(
      */
 
     for (childPtr = winPtr->childList; childPtr != NULL;
-				       childPtr = childPtr->nextPtr) {
+	    childPtr = childPtr->nextPtr) {
 	if (!Tk_IsMapped(childPtr) || Tk_IsTopLevel(childPtr)) {
 	    continue;
 	}
-
-	GenerateUpdates(updateRgn, childPtr);
+	GenerateUpdates(updateRgn, updateBounds, childPtr);
     }
 
     /*
@@ -567,13 +585,12 @@ GenerateUpdates(
     if (Tk_IsContainer(winPtr)) {
 	childPtr = TkpGetOtherWindow(winPtr);
 	if (childPtr != NULL && Tk_IsMapped(childPtr)) {
-	    GenerateUpdates(updateRgn, childPtr);
+	    GenerateUpdates(updateRgn, updateBounds, childPtr);
 	}
 
 	/*
-	 * NOTE: Here we should handle out of process embedding.
+	 * TODO: Here we should handle out of process embedding.
 	 */
-
     }
 
     return 1;
@@ -922,9 +939,18 @@ TkWmProtocolEventProc(
  */
 
 int
-Tk_MacOSXIsAppInFront (void)
+Tk_MacOSXIsAppInFront(void)
 {
-    return tkMacOSXAppInFront;
+    OSStatus err;
+    ProcessSerialNumber frontPsn, ourPsn = {0, kCurrentProcess};
+    Boolean isFrontProcess = true;
+
+    err = ChkErr(GetFrontProcess, &frontPsn);
+    if (err == noErr) {
+	ChkErr(SameProcess, &frontPsn, &ourPsn, &isFrontProcess);
+    }
+    
+    return (isFrontProcess == true);
 }
 
 /*
@@ -943,13 +969,23 @@ Tk_MacOSXIsAppInFront (void)
  *----------------------------------------------------------------------
  */
 static void
-ClearPort(CGrafPtr port) {
+ClearPort(
+    CGrafPtr port,
+    HIShapeRef updateRgn)
+{
     CGContextRef context;
     Rect bounds;
     CGRect rect;
 
     GetPortBounds(port, &bounds);
     QDBeginCGContext(port, &context);
+    SyncCGContextOriginWithPort(context, port);
+    CGContextConcatCTM(context, CGAffineTransformMake(1.0, 0.0, 0.0, -1.0, 0.0,
+	    bounds.bottom - bounds.top));
+    if (updateRgn) {
+	ChkErr(HIShapeReplacePathInCGContext, updateRgn, context);
+	CGContextEOClip(context);
+    }
     rect = CGRectMake(0, 0, bounds.right, bounds.bottom);
     CGContextClearRect(context, rect);
     QDEndCGContext(port, &context);
